@@ -7,18 +7,21 @@ from .type_utils import is_dict_type, is_list_type
 # Helper functions
 # =============================================================================
 
+
 def _get_required_arg(args, name):
     try:
         return args.pop(0)
     except IndexError:
         raise CrazyError(f'Missing argument: {name}') from None
 
+
 def _get_optional_arg(args, default=None):
     try:
         return args.pop(0)
     except IndexError:
         return default
 
+
 def _require_no_more(args):
     if args:
         raise CrazyError(f'Too many arguments: {args}')
@@ -27,12 +30,15 @@ def _require_no_more(args):
 # Command validation functions
 # =============================================================================
 
+
 def _validate_none(_args):
     pass
 
+
 def _validate_void(args):
     _require_no_more(args)
 
+
 def _validate_choices(args):
     choices = _get_required_arg(args, 'values')
     _require_no_more(args)
@@ -53,6 +59,7 @@ def _validate_choices(args):
     else:
         raise CrazyError('values: Not a list or dictionary')
 
+
 def _validate_file(args):
     opts = _get_optional_arg(args, {})
     _require_no_more(args)
@@ -70,6 +77,7 @@ def _validate_file(args):
         else:
             raise CrazyError(f'Unknown option: {key}')
 
+
 def _validate_range(args):
     start = _get_required_arg(args, "start")
     stop  = _get_required_arg(args, "stop")
@@ -94,6 +102,7 @@ def _validate_range(args):
     else:
         raise CrazyError("step: cannot be 0")
 
+
 def _validate_exec(args):
     cmd = _get_required_arg(args, 'command')
     _require_no_more(args)
@@ -101,6 +110,7 @@ def _validate_exec(args):
     if not isinstance(cmd, str):
         raise CrazyError(f"Command is not a string: {cmd}")
 
+
 def _validate_value_list(args):
     opts = _get_required_arg(args, 'options')
     _require_no_more(args)
@@ -143,6 +153,7 @@ def _validate_value_list(args):
     if len(separator) != 1:
         raise CrazyError(f'Invalid length for separator: {separator}')
 
+
 def _validate_combine(args):
     commands = _get_required_arg(args, 'commands')
     _require_no_more(args)
@@ -171,10 +182,20 @@ def _validate_combine(args):
     if len(commands) == 1:
         raise CrazyError('commands: Must contain more than one command')
 
+
+def _validate_history(args):
+    pattern = _get_required_arg(args, 'pattern')
+    _require_no_more(args)
+
+    if not isinstance(pattern, str):
+        raise CrazyError(f"Pattern is not a string: {pattern}")
+
+
 # =============================================================================
 # Real validation functions
 # =============================================================================
 
+
 def validate_complete(complete):
     '''Validate a completion command.'''
 
@@ -209,6 +230,7 @@ def validate_complete(complete):
         'exec_internal': _validate_exec,
         'value_list':    _validate_value_list,
         'combine':       _validate_combine,
+        'history':       _validate_history,
         # Bonus
         'mountpoint':    _validate_void,
         'net_interface': _validate_void,
@@ -225,6 +247,7 @@ def validate_complete(complete):
 
     validate_commands[command](args)
 
+
 def validate_commandline(cmdline):
     '''Validate completion commands of options/positionals in a commandline.'''
 
@@ -247,6 +270,7 @@ def validate_commandline(cmdline):
                 positional.metavar,
                 e)) from e
 
+
 def validate_commandlines(cmdline):
     '''Validate completion commands of options/positionals in all commandlines.'''
 
