@@ -3,6 +3,7 @@
 from . import shell
 from . import helpers
 from .type_utils import is_dict_type
+from .str_utils import indent, join_with_wrap
 
 CHOICES_INLINE_THRESHOLD = 80
 # The `choices` command can in Fish be expressed inline in a complete command
@@ -14,6 +15,7 @@ CHOICES_INLINE_THRESHOLD = 80
 # This variable defines how big this string can get before a function
 # is used instead.
 
+
 class FishCompletionBase:
     '''Base class for Fish completions.'''
 
@@ -30,6 +32,7 @@ class FishCompletionBase:
         '''Return the code that can be used for completing an argument.'''
         raise NotImplementedError
 
+
 class FishCompleteNone(FishCompletionBase):
     '''Class for completing an argument without a completer.'''
 
@@ -39,10 +42,11 @@ class FishCompleteNone(FishCompletionBase):
     def get_code(self):
         return ''
 
+
 class FishCompletionCommand(FishCompletionBase):
     '''Class for executing a command and parsing its output.'''
 
-    def  __init__(self, command):
+    def __init__(self, command):
         self.command = command
 
     def get_code(self):
@@ -51,6 +55,7 @@ class FishCompletionCommand(FishCompletionBase):
     def get_args(self):
         return ['-f', '-a', '(%s)' % self.command]
 
+
 class FishCompleteChoices(FishCompletionBase):
     '''Class for completing choices.'''
 
@@ -71,8 +76,8 @@ class FishCompleteChoices(FishCompletionBase):
     @staticmethod
     def _get_code_for_list(choices):
         code = "printf '%s\\n' \\\n"
-        for item in choices:
-            code += '  %s \\\n' % shell.escape(str(item))
+        escaped = [shell.escape(str(item)) for item in choices]
+        code += indent(join_with_wrap(' ', ' \\\n', 80, escaped), 2)
         return code.rstrip(' \\\n')
 
     @staticmethod
@@ -92,9 +97,8 @@ class FishCompleteChoices(FishCompletionBase):
             return ['-f', '-a', arg]
 
         code = self.get_code()
-        funcname = self.ctxt.helpers.get_unique_function_name(self.ctxt)
-        self.ctxt.helpers.add_function(helpers.FishFunction(funcname, code))
-        funcname = self.ctxt.helpers.use_function(funcname)
+
+        funcname = self.ctxt.helpers.add_dynamic_func(self.ctxt, code)
         return ['-f', '-a', '(%s)' % funcname]
 
     def get_code(self):
@@ -142,13 +146,13 @@ class FishCompleteFileDir(FishCompletionBase):
 
         return '__fish_complete_directories'
 
+
 class FishCompleteValueList(FishCompletionBase):
     '''Class for completing a list of values.'''
 
     def __init__(self, ctxt, opts):
         separator = opts.get('separator', ',')
         values = opts['values']
-        funcname = ctxt.helpers.get_unique_function_name(ctxt)
 
         if is_dict_type(values):
             code = "printf '%s\\t%s\\n' \\\n"
@@ -161,9 +165,7 @@ class FishCompleteValueList(FishCompletionBase):
                 code += '  %s \\\n' % shell.escape(value)
             code = code.rstrip(' \\\n')
 
-        ctxt.helpers.add_function(helpers.FishFunction(funcname, code))
-        funcname = ctxt.helpers.use_function(funcname)
-
+        funcname = ctxt.helpers.add_dynamic_func(ctxt, code)
         self.cmd = '__fish_complete_list %s %s' % (shell.escape(separator), funcname)
 
     def get_args(self):
@@ -172,6 +174,7 @@ class FishCompleteValueList(FishCompletionBase):
     def get_code(self):
         return self.cmd
 
+
 class FishCompleteCombine(FishCompletionBase):
     '''Used for combining multiple complete commands.'''
 
@@ -194,9 +197,9 @@ class FishCompleteCombine(FishCompletionBase):
             return ['-f', '-a', '(%s)' % '; '.join(self.code)]
 
         code = '\n'.join(self.code)
-        funcname = self.ctxt.helpers.get_unique_function_name(self.ctxt)
-        self.ctxt.helpers.add_function(helpers.FishFunction(funcname, code))
-        return ['-f', '-a', '(%s)' % self.ctxt.helpers.use_function(funcname)]
+        funcname = self.ctxt.helpers.add_dynamic_func(self.ctxt, code)
+        return ['-f', '-a', '(%s)' % funcname]
+
 
 class FishCompleter(shell.ShellCompleter):
     '''Code generator for completing arguments in Fish.'''
@@ -270,6 +273,10 @@ class FishCompleter(shell.ShellCompleter):
     def combine(self, ctxt, commands):
         return FishCompleteCombine(ctxt, self, commands)
 
+    def history(self, ctxt, pattern):
+        func = ctxt.helpers.use_function('history')
+        return FishCompletionCommand('%s %s' % (func, shell.escape(pattern)))
+
     # =========================================================================
     # Bonus
     # =========================================================================
