@@ -4,16 +4,19 @@ import sys
 from collections import namedtuple
 from .type_utils import is_list_type
 
+
 def warn(*a):
     '''Print a warning.'''
 
     print('WARNING:', *a, file=sys.stderr)
 
+
 def print_err(*a):
     '''Print to STDERR.'''
 
     print(*a, file=sys.stderr)
 
+
 class GeneralAbbreviationGenerator:
     '''A class for generating abbreviations from a list of words.
 
@@ -90,6 +93,7 @@ class GeneralAbbreviationGenerator:
             r.extend(self.get_abbreviations(word))
         return r
 
+
 class OptionAbbreviationGenerator(GeneralAbbreviationGenerator):
     '''AbbreviationGenerator for abbreviating long and old-style options.'''
 
@@ -109,12 +113,14 @@ class OptionAbbreviationGenerator(GeneralAbbreviationGenerator):
 
         super().__init__(3, words)
 
+
 class CommandAbbreviationGenerator(GeneralAbbreviationGenerator):
     '''AbbreviationGenerator for abbreviating commands.'''
 
     def __init__(self, words):
         super().__init__(1, words)
 
+
 class DummyAbbreviationGenerator:
     '''A dummy abbreviation generator that returns the original word as the abbreviation.
 
@@ -140,6 +146,7 @@ class DummyAbbreviationGenerator:
 
         return words
 
+
 def get_option_abbreviator(commandline):
     '''Return an OptionAbbreviationGenerator for options in `commandline`.'''
 
@@ -155,6 +162,7 @@ def get_option_abbreviator(commandline):
 
     return OptionAbbreviationGenerator(option_strings)
 
+
 def get_all_command_variations(commandline):
     '''Return all possible names for this command.
 
@@ -174,24 +182,66 @@ def get_all_command_variations(commandline):
 
     return abbrevs.get_abbreviations(commandline.prog) + commandline.aliases
 
+
 def get_defined_option_types(commandline):
     '''Return a tuple of defined option types.'''
 
-    old   = False
-    long  = False
-    short = False
+    short_required = False
+    short_optional = False
+    short_flag     = False
+    long_required  = False
+    long_optional  = False
+    long_flag      = False
+    old_required   = False
+    old_optional   = False
+    old_flag       = False
+    positionals    = False
 
     for cmdline in commandline.get_all_commandlines():
+        if len(cmdline.get_positionals()) > 0:
+            positionals = True
+
+        if cmdline.get_subcommands() and len(cmdline.get_subcommands().subcommands) > 0:
+            positionals = True
+
         for option in cmdline.options:
-            for option_string in option.option_strings:
-                if option_string.startswith('--'):
-                    long = True
-                elif len(option_string) == 2:
-                    short = True
+            if option.get_long_option_strings():
+                if option.complete and option.optional_arg is True:
+                    long_optional = True
+                elif option.complete:
+                    long_required = True
                 else:
-                    old = True
+                    long_flag = True
+
+            if option.get_old_option_strings():
+                if option.complete and option.optional_arg is True:
+                    old_optional = True
+                elif option.complete:
+                    old_required = True
+                else:
+                    old_flag = True
+
+            if option.get_short_option_strings():
+                if option.complete and option.optional_arg is True:
+                    short_optional = True
+                elif option.complete:
+                    short_required = True
+                else:
+                    short_flag = True
+
+    return namedtuple('Types', (
+        'positionals',
+        'short_required', 'short_optional', 'short_flag', 'short',
+        'long_required',  'long_optional',  'long_flag',  'long',
+        'old_required',   'old_optional',   'old_flag',   'old'))(
+        positionals,
+        short_required,    short_optional,   short_flag,
+        short_required or  short_optional or short_flag,
+        long_required,     long_optional,    long_flag,
+        long_required  or  long_optional  or long_flag,
+        old_required,      old_optional,     old_flag,
+        old_required   or  old_optional   or old_flag)
 
-    return namedtuple('Types', ('short', 'long', 'old'))(short, long, old)
 
 def get_query_option_strings(commandline, with_parent_options=True):
     '''Return a string that can be used by {fish,zsh}_query functions.
@@ -212,6 +262,7 @@ def get_query_option_strings(commandline, with_parent_options=True):
 
     return ','.join(r)
 
+
 def is_worth_a_function(commandline):
     '''Check if a commandline "is worth a function".
 
